﻿// Program.cs : main project file.
/*!
** NAME: \file  Program.cs              									\n
**																			\n
** (C) Copyright PREHKEYTEC 2019.      									    \n
**																			\n
** PURPOSE:																	\n
**  This module contains LCD routines for C# PKTUSB API demonstration.  	\n
**																			\n
** No Warranty																\n
** The Software is being delivered to you "AS IS" and PrehKeyTec GmbH		\n
** makes no warranty as to its use or performance. Details see readme.		\n
**																			\n
** The PKTUSBAPI is written in c# (managed code).							\n
**																			\n
**  \brief This console C# demo works with PKTUSBAPI.			   	        \n
**																		   	\n
** FUNCTIONS:																\n
**  Main routine													        \n
**  OpenDevice												                \n
**  myAPI_OnDeviceStatusChanged												\n
**	myDevice_OnReceivedPOSkey 												\n
**  StartProgress                                                           \n
**  StartAlert                                                              \n
**  Reset                                                                   \n
**																			\n
** \author          T Meusert         	\version       5.1				    \n
**																			\n
**   									\date   	   01 Oct 2019			\n
**																			\n
** REVISION HISTORY:														\n
**																			\n
**  DATE       	VERSION	PROGRAMMER     	DESCRIPTION							\n
**  ========== 	======= ============== 	=======================		        \n
**  01. Oct 19  5.1  	Meusert T.   	Module created 						\n
*/

// INCLUDES
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

// Reference to PKTUSBAPI.DLL The Path to the DLL should be relative to Demoapplication
// With a post built command Xcopies the required DLL from Demoapplication to target folders
using PKTUSBAPI;
using System.Threading;

namespace PKTUSBAPI_LCD_Demo
{
    class Program
    {
        private static PKTUSBAPI_Class myAPI = null;
        private static USB_Device myDevice = null;
        private static Thread thStartProgress = null;
        private static Thread thStartAlert = null;
        
        private static int POSKEY_START = 1; // Define POS-Key to start the progress
        private static int POSKEY_STOP = 2;  // Define POS-Key to stop the alert
        private static int POSKEY_EXIT = 3;  // Define POS-Key to exit the demo application

        static void Main(string[] args)
        {
            try
            {
                //Initialize the PKTUSBAPI (Show all existing PrehKeyTec USB Devices) 
                Console.WriteLine("\nInitialize the PKTUSBAPI");
                myAPI = new PKTUSBAPI_Class();

                //Show the library versions 
                Console.WriteLine("PKTUSBAPI.dll Version: " + myAPI.VersionAPI);
                Console.WriteLine("PKTUSBxx.dll Version: " + myAPI.VersionDLL);

                //Set up on device status changed event
                Console.WriteLine("\nSet up Event Handler: OnDeviceStatusChanged");
                myAPI.OnDeviceStatusChanged += new EventHandler<EventArguments.OnDeviceStatusChanged>(myAPI_OnDeviceStatusChanged);

                // Open connected device
                Console.WriteLine("\nSearching for PrehKeyTec devices...");
                OpenDevice();
                
                // wait for any key to quit
                Console.ReadKey(); 
            }
            catch (Exception e)
            {
                Console.WriteLine("\nEXIT WITH ERROR: " + e.Message);
            }
        }

        static void myAPI_OnDeviceStatusChanged(object sender, EventArguments.OnDeviceStatusChanged e)
        {
            //Do something only if a device was attached or detached
            if (e.ConnectedStatusChanged)
            {
                //Check if myDevice was detached
                if (myDevice != null && e.ChangedDevice.DevicePath.Equals(myDevice.DevicePath) && !e.ChangedDevice.IsConnected)
                {
                    //Close myDevice instance
                    Console.WriteLine("\nDevice closed");
                    myDevice.Close(); // device close
                    myDevice = null;
                }
                //Searching for a device to open if myDevice is 'null'
                if (myDevice == null) OpenDevice(); // try to open new device
            }
        }
        
        static void OpenDevice()
        {
            //Open the device that was found first from the API
            myDevice = myAPI.OpenDevice();
            if (myDevice != null)
            {
                //Display some device information
                Console.WriteLine("\n+  +  +  +  +  +  +  +  +  +  +  +  +  +");
                Console.WriteLine("Device opened:");
                Console.WriteLine(myDevice.DevicePath + "\n");
                Console.WriteLine(myDevice.VersionString);
                //Set up on received POS key event for my device
                Console.WriteLine("Set up Event Handler: OnReceivedPOSkey");
                myDevice.OnReceivedPOSkey += new EventHandler<EventArguments.OnReceivedPOSkey>(myDevice_OnReceivedPOSkey);
                Console.WriteLine("+  +  +  +  +  +  +  +  +  +  +  +  +  +");
                //Reset LCD and LEDs
                Reset();
            }
        }

        static void myDevice_OnReceivedPOSkey(object sender, EventArguments.OnReceivedPOSkey e)
        {
            if (e.Pressed)
            {
                if (e.Key == POSKEY_START)
                {
                    // Start Progress Thread
                    Console.WriteLine("POS-Key " + e.Key + " pressed:   Progress thread started");
                    if ((thStartProgress == null || !thStartProgress.IsAlive) && (thStartAlert == null || !thStartAlert.IsAlive))
                    {
                        thStartProgress = new Thread(StartProgress);
                        thStartProgress.Start();
                    }
                }
                else if (e.Key == POSKEY_STOP)
                {
                    // Stop alert
                    Console.WriteLine("POS-Key " + e.Key + " pressed:   Alert canceled");
                    Reset();
                }
                else if (e.Key == POSKEY_EXIT)
                {
                    // Exit application
                    Console.WriteLine("POS-Key " + e.Key + " pressed:   Exit application");
                    Reset();
                    if (myDevice != null)
                    {
                        myDevice.ClearLCDText();
                        myDevice.Close();
                    }
                    Environment.Exit(0);
                }
            }            
        }

        static void StartProgress()
        {
            if (myDevice != null)
            {
                // Start Accept LED flashing orange 
                myDevice.SetLEDAcceptOnly(Constants.ACCEPT_GREEN, Constants.ACCEPT_FLASH_800ms);
                // Show Progress on LCD display
                myDevice.ClearLCDText();
                myDevice.SetLCDTextLine1("Please wait...");
                for (int i = 20; i < 40; i++)
                {
                    myDevice.SetLCDTextAt(new byte[]{0xff}, i);
                    Thread.Sleep(200);
                }
                // Start Alert Thread
                if (thStartAlert == null || !thStartAlert.IsAlive)
                {
                    thStartAlert = new Thread(StartAlert);
                    thStartAlert.Start();
                }                
            }            
        }
        
        static void StartAlert()
        {
            if (myDevice != null)
            {
                myDevice.SetLCDText("Press POS-Key " + POSKEY_STOP, "to stop the alert.");
                // Start Accept LED flashing red 
                myDevice.SetLEDAcceptOnly(Constants.ACCEPT_RED, Constants.ACCEPT_FLASH_400ms);
                // Init alert tones
                myDevice.InitSound(2500, 3000, 3500, 30, 30, 30);

                bool state = false;
                int frequency = 400; // alert frequency in ms
                while (true)
                {
                    state = !state;
                    if (state)
                    {
                        myDevice.SetLCDBrightness(8);           // Set display brightness to maximum
                        myDevice.PlayTone1_Duration(frequency); // Play tone 1
                    }
                    else
                    {
                        myDevice.SetLCDBrightness(1);           // Set display brightness to minimum
                        myDevice.PlayTone2_Duration(frequency); // Play tone 2
                    }
                    Thread.Sleep(frequency);
                }
            }  
        }

        static void Reset()
        {
            if (myDevice != null)
            {
                // Cancel all threads
                if (thStartAlert != null) thStartAlert.Abort();
                if (thStartProgress != null) thStartProgress.Abort();
                // Reset the Accept LED
                myDevice.ResetLEDs();
                // Reset LCD Display
                myDevice.SetLCDContrast(40); // Default Value = 40;
                myDevice.SetLCDBrightness(8); // 1 = Minimum, 8 = Maximum
                myDevice.ClearLCDText();
                myDevice.SetLCDText("Press POS-Key " + POSKEY_START, "to start the demo.");
            }  
        }
    }
}
